/**
  Google Reader Notifier
  By Mark D.B.D <markdbd@gmail.com>
  http://www.markdbd.com
**/

var gGrAjaxDebug = false;

var GrAjax = {

	/**
	* @var mixed Objeto xmlhttp
	* @access public
	*/

	xmlhttp: false,

	/**
	* @var string Metodo, get o post
	* @access public
	*/

	method: null,

	/**
	* @var string Datos a enviar
	* @access public
	*/

	data: null,

	/**
	* @var array {'onLoad': funcion, 'onError': funcion}
	* @access public
	*/

	functionCall: null,

	/**
	* @var string Respuesta recibida
	* @access public
	*/

	response: null,

	/**
	* @var string Tipo de respuesta
	* @access public
	*/

	responseType: 'xml',

	/**
	* Inicializa xmlhttp
	*
	* @return void
	* @access public
	*/

	init: function()
	{
		try
		{
			GrAjax.xmlhttp = new XMLHttpRequest();
		}
		catch(eee)
		{
			GrAjax.xmlhttp = false;
		}

		if(GrAjax.xmlhttp)
		{
			showMsg("xmlhttp inicializado con exito", gGrAjaxDebug);
		}
		else
		{
			showMsg("xmlhttp inicializado sin exito", gGrAjaxDebug);
		}
	},

	/**
	* Envia petición al xmlhttp
	*
	* @param string Url donde realizar la petición
	* @param string Metodo, get o post
	* @param string En caso de post pasaremos los datos a enviar
	* @param string Función a llamar al finalizar la petición
	* @param string Tipo de respuesta, texto, xml o json
	* @return void
	* @access public
	*/

	sendRequest: function(url, method, data, functionCall, responseType)
	{
		if(!GrAjax.xmlhttp)
		{
			return;
		}

		GrAjax.method = (method)?method:'get';
		GrAjax.data = (data)?data:null;
		GrAjax.functionCall = functionCall;
		GrAjax.response = null;
		GrAjax.responseType = (responseType)?responseType:'text';

		showMsg('Url: ' + url + '\nMetodo: ' + GrAjax.method + '\nDatos a enviar: ' + GrAjax.data + '\nFuncion a llamar onload: ' + GrAjax.functionCall['onLoad'] + '\nFuncion a llamar onerror: ' + GrAjax.functionCall['onError'] + '\nTipo de respuesta: ' + GrAjax.responseType, gGrAjaxDebug);

		//Realizamos la petición
		GrAjax.xmlhttp.onreadystatechange = GrAjax.handleResponse;
		GrAjax.xmlhttp.open(GrAjax.method, url, true);

		GrAjax.xmlhttp.setRequestHeader('User-Agent','GoogleReaderNotifier/Google Reader Notifier for Firefox');

		if (GrAjax.method.toLowerCase() == "post")
		{
			GrAjax.xmlhttp.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
		}

		GrAjax.xmlhttp.send(GrAjax.data);
	},

	/**
	* Resultado petición
	*
	* @return void
	* @access public
	*/

	handleResponse: function()
	{

		showMsg('Estado: ' + GrAjax.xmlhttp.readyState, gGrAjaxDebug);

		if (GrAjax.xmlhttp.readyState == 4)
		{
			if(!GrAjax.xmlhttp.responseText)
			{
				GrAjax.functionCall['onError']();
			}
			else if (GrAjax.xmlhttp.status == 200)
			{
				showMsg("Respuesta recibida: " + GrAjax.xmlhttp.responseText, gGrAjaxDebug);
				switch(GrAjax.responseType)
				{
					case 'xml':
						GrAjax.response = GrAjax.xmlhttp.responseXML;
					break;

					case 'json':
						GrAjax.response = JSON.parse(GrAjax.xmlhttp.responseText);
					break;

					default:
						GrAjax.response = GrAjax.xmlhttp.responseText;
				}

				GrAjax.functionCall['onLoad']();
			}
			else
			{
				showMsg("Error con estado: " + GrAjax.xmlhttp.status, gGrAjaxDebug);
				GrAjax.functionCall['onError']();
			}
		}
	}
}

/*
    The global object JSON contains two methods.

    JSON.stringify(value) takes a JavaScript value and produces a JSON text.
    The value must not be cyclical.

    JSON.parse(text) takes a JSON text and produces a JavaScript value. It will
    return false if there is an error.
*/
var JSON = function () {
    var m = {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        s = {
            'boolean': function (x) {
                return String(x);
            },
            number: function (x) {
                return isFinite(x) ? String(x) : 'null';
            },
            string: function (x) {
                if (/["\\\x00-\x1f]/.test(x)) {
                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
                        var c = m[b];
                        if (c) {
                            return c;
                        }
                        c = b.charCodeAt();
                        return '\\u00' +
                            Math.floor(c / 16).toString(16) +
                            (c % 16).toString(16);
                    });
                }
                return '"' + x + '"';
            },
            object: function (x) {
                if (x) {
                    var a = [], b, f, i, l, v;
                    if (x instanceof Array) {
                        a[0] = '[';
                        l = x.length;
                        for (i = 0; i < l; i += 1) {
                            v = x[i];
                            f = s[typeof v];
                            if (f) {
                                v = f(v);
                                if (typeof v == 'string') {
                                    if (b) {
                                        a[a.length] = ',';
                                    }
                                    a[a.length] = v;
                                    b = true;
                                }
                            }
                        }
                        a[a.length] = ']';
                    } else if (x instanceof Object) {
                        a[0] = '{';
                        for (i in x) {
                            v = x[i];
                            f = s[typeof v];
                            if (f) {
                                v = f(v);
                                if (typeof v == 'string') {
                                    if (b) {
                                        a[a.length] = ',';
                                    }
                                    a.push(s.string(i), ':', v);
                                    b = true;
                                }
                            }
                        }
                        a[a.length] = '}';
                    } else {
                        return;
                    }
                    return a.join('');
                }
                return 'null';
            }
        };
    return {
        copyright: '(c)2005 JSON.org',
        license: 'http://www.crockford.com/JSON/license.html',
/*
    Stringify a JavaScript value, producing a JSON text.
*/
        stringify: function (v) {
            var f = s[typeof v];
            if (f) {
                v = f(v);
                if (typeof v == 'string') {
                    return v;
                }
            }
            return null;
        },
/*
    Parse a JSON text, producing a JavaScript value.
    It returns false if there is a syntax error.
*/
        parse: function (text) {
            try {
                return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(
                        text.replace(/"(\\.|[^"\\])*"/g, ''))) &&
                    	eval('(' + text + ')');
            } catch (e) {
                return false;
            }
        }
    };
}();
