/**
  Google Reader Notifier
  By Mark D.B.D <markdbd@gmail.com>
  http://www.markdbd.com
**/

var gGrNotifyUpdate = 'updateInfo';

var gGrGoogleReaderUrl = 'http://www.google.com/reader';
var gGrGoogleReaderCompareUrl = 'http://www.google.com/reader/view/';
var gGrGoogleReaderFindSubscribeUrl = 'http://www.google.com/reader/finder?q=';
var gGrGoogleReaderNotifierUrl = 'http://www.markdbd.com/proyectos/google_reader_notifier';

var gGrErrorStatus = '00';
var gGrLoadingStatus = '01';
var gGrNoItemsStatus = '10';
var gGrNewItemsStatus = '11';

var gGrClickCurrentTab = 0;
var gGrClickNewTab = 1;
var gGrClickUnfocusedTab = 2;
var gGrClickNewWindow = 3;
var gGrClickUpdate = 4;
var gGrClickPreferences = 5;
var gGrClickShowMenu = 6;

var gGrDebug = false;
var gGrVersion = '0.36';

var Gr =
{
	/**
	* @var int Número de elementos sin leer
	*/

	countItems: null,

	/**
	* @var mixed Etiquetas del usuario
	*/

	labels: null,

	/**
	* @var mixed Propiedades
	*/

	properties: null,

	/**
	* @var int Indica el timer actual
	*/

	timer: null,

	/**
	* @var mixed Observer
	*/

	observer: null,

	/**
	* Constructor
	*
	* @return void
	*/

	load: function()
	{
		//Inicializamos

		GrAjax.init();

		Gr.properties = $('gr-properties');
		Gr.countItems = 0;

		//Añadimos observer para las preferencias

		Gr.observer = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		Gr.observer.addObserver(Gr, gGrNotifyUpdate, false);

		//Cambiamos los iconos

		Gr.changeIcon(gGrNoItemsStatus);

		//Evento para el menu contextual

		$('contentAreaContextMenu').addEventListener('popupshowing', Gr.contextMenu,false);

		//Si la última versión es diferente a la instalada le mostramos los cambios realizados

		if(GrPref.prefs.getCharPref('current_version') != gGrVersion)
		{
			GrPref.prefs.setCharPref('current_version', gGrVersion);
			window.setTimeout('Gr.openUrl(gGrGoogleReaderNotifierUrl+"#changelog");Gr.showPreferences()', 1000);
		}
		else
		{
			window.setTimeout('Gr.update()', 3000);
		}
	},

	/**
	* Destructor
	*
	* @return void
	*/

	unload: function()
	{
		Gr.observer.removeObserver(Gr, gGrNotifyUpdate);

		if(!GrPref.prefs.getBoolPref('remember'))
		{
			GrPref.setPassword('');
		}
	},

	/**
	* Observer para las preferencias
	*
	* @return void
	*/

	observe: function(aSubject, aTopic, aData)
	{
	    if (aTopic == gGrNotifyUpdate)
    	{
			window.clearTimeout(Gr.timer);
			Gr.timer = window.setTimeout('Gr.update()', 1000);
	    }
	},

	/*
	 * Comprueba si es la misma cuenta, ya que a lo mejor otra extension ha cambiado el valor de la cookie
	 *
	 * @param string google SID
	 * @return bool
	 */

	sameGoogleAccount: function(SID)
	{
		if(SID == GrPref.prefs.getCharPref('logged_SID'))
		{
			return true;
		}
		else
		{
			return false;
		}
	},

	/*
	 * Devuelve el SID de la cookie
	 *
	 * @return string
	 */

	GetCurrentSID: function()
	{
		var cookieManager = Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager);
		var iter = cookieManager.enumerator;

		while (iter.hasMoreElements())
		{
			var cookie = iter.getNext();
			if (cookie instanceof Components.interfaces.nsICookie)
			{
				if ((cookie.host.indexOf(".google.", 0) != -1) && (cookie.path == '/'))
				{
					if(cookie.name == 'SID')
					{
						return cookie.value;
					}
				}
			}
		}
	},

	/*
	* Actualiza o logea
	*
	* @return void
	*/

	update: function()
	{
		Gr.labels = new Array();
		Gr.loading();

		if(Gr.sameGoogleAccount(Gr.GetCurrentSID()))
		{
			Gr.getItems();
		}
		else
		{
			Gr.login();
		}
	},

	/*
	* Login
	*
	* @return void
	*/

	login: function()
	{
		if(GrPref.accountExist())
		{

			var functionCall = 	{'onLoad':
									function()
									{
										if(GrAjax.response.indexOf('error', 0) == -1)
										{
											GrPref.prefs.setCharPref('logged_SID', Gr.GetCurrentSID());
											Gr.getItems();
										}
										else
										{
											Gr.errorReport('loginFailed');
										}
									},
									'onError':
									function()
									{
										Gr.errorReport('loginFailed');
									}
								};

			GrAjax.sendRequest( 'https://www.google.com/accounts/ServiceLoginAuth',
								'post',
								'Email='+encodeURIComponent(GrPref.prefs.getCharPref('usermail'))+'&Passwd='+encodeURIComponent(GrPref.getPassword())+'&service=reader&PersistentCookie=yes&continue=http://www.google.com/reader&nui=1',
								functionCall,
								'text'
							  );
		}
		else
		{
			Gr.errorReport('loginFailed');
		}
	},

	/*
	* Actualiza el contador
	*
	* @return void
	*/

	getItems: function()
	{
		window.clearTimeout(Gr.timer);

		var functionCall =	{'onLoad':
								function()
								{
									Gr.setNewItems(Gr.countNewItems(GrAjax.response.unreadcounts));

	  								try
	  								{
	  									var interval = GrPref.prefs.getIntPref('interval');
	  								}
									catch(e)
									{
										var interval = GrPref.defaultInterval;
									}

									Gr.timer = window.setTimeout('Gr.update()', interval * 1000);
								},
								'onError':
								function()
								{
									Gr.errorReport();
								}
							};

		GrAjax.sendRequest( 'https://www.google.com/reader/api/0/unread-count?all=true&output=json',
							'get',
							null,
							functionCall,
							'json'
						  );
	},

	/**
	* Cuenta el número de elementos sin leer
	*
	* @param mixed Elementos recibidos vía Ajax
	* @return int
	*/

	countNewItems: function(items)
	{
		var total = 0;
		var unorderLabels = Array();

		for(i in items)
		{
			if(items[i].count > 0)
			{
				if(items[i].id.indexOf('feed/http', 0) != -1)
				{
					total += items[i].count;
				}
				else if((GrPref.prefs.getBoolPref('show_unread_items_in_labels')) && (items[i].id.match(/^user\/\d{20}\/label\/\S{1,}$/)))
				{
					var label = items[i].id.substr(items[i].id.lastIndexOf('/') + 1, items[i].id.length);
					unorderLabels[label] = items[i].count;
				}
			}
		}

		if(GrPref.prefs.getBoolPref('show_unread_items_in_labels'))
		{
			Gr.setLabels(unorderLabels);
		}

		return total;
	},

	/**
	* Asigna y ordena las etiquetas
	*
	* @param mixed Array etiquetas desordenadas
	* @return mixed
	*/

	setLabels: function(unorderLabels)
	{
		for(i in unorderLabels)
		{
			var biggest = i;
			for(j in unorderLabels)
			{
				if(unorderLabels[j] > unorderLabels[biggest])
				{
					biggest = j;
				}
			}

			//Añadimos el + si son 100 los elementos por leer en la etiqueta

			if(unorderLabels[biggest] == 100)
			{
				unorderLabels[biggest] = '+100';
			}

			Gr.labels[biggest] = unorderLabels[biggest];
			unorderLabels[biggest] = null;
		}
	},

	/**
	* Marcar los nuevos elementos
	*
	* @param int Número de elementos sin leer
	* @return void
	*/

	setNewItems: function(unreadItems)
	{
		var oldItems = Gr.countItems;
		Gr.countItems = unreadItems;

		//Actualizamos el contador

		Gr.updateCounter();

		//Si ha cambiado el número de elementos por leer y son más de 0, mostramos la ventana de alerta

		if((oldItems != Gr.countItems) && ( Gr.countItems > 0))
		{
			Gr.alertWindow();
		}
	},

	/**
	* Muestra la ventana de notificación
	*
	* @return void
	*/

	alertWindow: function()
	{
		//Si tiene la notificación activada y no es una ventana popup, la mostramos

		if(GrPref.prefs.getBoolPref('notification'))
		{
			var listener = {
			  observe: function(subject, topic, data)
			  {
			    if(topic == 'alertclickcallback')
			    {
					Gr.markAsRead();
					getBrowser().selectedTab = getBrowser().addTab(gGrGoogleReaderUrl);
				}
			  }
			}

			var alertsService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);

			alertsService.showAlertNotification("chrome://google_reader_notifier/skin/" + GrPref.prefs.getCharPref('icon_theme') + "/logo.png",
			                                    'Google Reader Notifier', Gr.properties.getFormattedString('unreadItems',[Gr.countItems]),
            			                        true, "", listener);
		}
	},

	/*
	* Actualiza el contador en la barra de estado
	*
	* @return void
	*/

	updateCounter: function()
	{

		//Creamos el tooltip especifico

		Gr.cleanTooltip();

		var totalCount = document.createElement("description");
		totalCount.setAttribute("value", (!Gr.countItems) ? Gr.properties.getString('allRead') : Gr.properties.getFormattedString('unreadItems',[Gr.countItems]));
		totalCount.setAttribute("style", (Gr.countItems) ? 'font-weight: bold' : '');
		$('gr-tooltip').appendChild(totalCount);

		if(GrPref.prefs.getBoolPref('show_unread_items_in_labels'))
		{
			for(label in Gr.labels)
			{
				var labelText = document.createElement("description");
				labelText.setAttribute("value", Gr.properties.getFormattedString('unreadItemsLabel',[Gr.labels[label],label]));
				$('gr-tooltip').appendChild(labelText);
			}
		}

		//Actualizamos la barra de estado

		$('gr-unread-items').value = Gr.countItems;
		$('gr-unread-items').hidden = GrPref.prefs.getBoolPref('hide_counter');
		Gr.changeIcon((!Gr.countItems) ? gGrNoItemsStatus : gGrNewItemsStatus);
	},

	/**
	* Marcar todos como leidos
	*
	* @return void
	*/

	markAsRead: function()
	{
		if(GrPref.prefs.getBoolPref('mark_as_read'))
		{
			Gr.countItems = 0;
			Gr.labels = null;
			Gr.updateCounter();
		}
	},

	/**
	* Función que se usa para reportar errores
	*
	* @param string Indice de la cadena
	* @return void
	*/

	errorReport: function(key)
	{
		Gr.countItems = 0;
		Gr.labels = null;
		Gr.changeIcon(gGrErrorStatus);

		if(key)
		{
			Gr.setTextTooltip(Gr.properties.getString(key));
		}

		$('gr-unread-items').value = Gr.countItems;
	},

	/**
	* Cargando
	*
	* @return void
	*/

	loading: function()
	{
		Gr.setTextTooltip(Gr.properties.getString('loading'));
		Gr.changeIcon(gGrLoadingStatus);
	},

	/*
	* Limpia el tooltip
	*
	* @return void
	*/

	cleanTooltip: function()
	{
		while($('gr-tooltip').hasChildNodes())
		{
			$('gr-tooltip').removeChild($('gr-tooltip').firstChild);
		}
	},

	/*
	* Cambia el texto del tooltip
	*
	* @param string Nuevo texto del tooltip
	* @return void
	*/

	setTextTooltip: function(text)
	{
			Gr.cleanTooltip();

			var tooltipText = document.createElement("description");
			tooltipText.setAttribute("value", text);

			$('gr-tooltip').appendChild(tooltipText);
	},

	/**
	* Función que gestiona los eventos del ratón
	*
	* @return void
	*/

	mouseClick: function(event)
	{
		if(GrPref.accountExist())
		{
			var clickAction = null;

	        if (event)
			{
	            if ((event.button == 0) && (!event.ctrlKey && !event.shiftKey))
				{
	                clickAction = GrPref.prefs.getIntPref('left_click');
	            }
				else if ((event.button == 1 ) || (event.ctrlKey))
				{
	                clickAction = GrPref.prefs.getIntPref('middle_click');
	            }
				else if ((event.button == 2 ) || (event.shiftKey))
				{
					clickAction = GrPref.prefs.getIntPref('right_click');
	            }

	            switch(clickAction)
				{
					case gGrClickCurrentTab:
						Gr.markAsRead();
						if(!Gr.openAndReuseOneTabPerURL(gGrGoogleReaderCompareUrl, true))
						{
							getBrowser().loadURI(gGrGoogleReaderUrl);
						}
					break;

					case gGrClickNewTab:
						Gr.markAsRead();
						if(!Gr.openAndReuseOneTabPerURL(gGrGoogleReaderCompareUrl, true))
						{
							getBrowser().selectedTab = getBrowser().addTab(gGrGoogleReaderUrl);
						}
					break;

					case gGrClickUnfocusedTab :
						Gr.markAsRead();
						if(!Gr.openAndReuseOneTabPerURL(gGrGoogleReaderCompareUrl, true))
						{
							getBrowser().addTab( gGrGoogleReaderUrl );
						}
					break;

					case gGrClickNewWindow:
						Gr.markAsRead();
						window.open(gGrGoogleReaderUrl);
					break;

					case gGrClickPreferences:
						Gr.showPreferences();
					break;

					case gGrClickUpdate:

						//Si no esta cargando puede actualizar

						if($('gr-icon').getAttribute('status') != gGrLoadingStatus)
						{
							Gr.update();
						}
					break;

					case gGrClickShowMenu:
						$('gr-popup').showPopup($('gr-status'),event.screenX + 6, event.screenY + 6, "popup", "bottomright", "bottomright");
					break;

				}
	        }
		}
		else
		{
			Gr.showPreferences();
		}
	},

	/**
	* Función que modifica el menu contextual
	*
	* @return void
	*/

	contextMenu: function()
	{
		if(GrPref.prefs.getBoolPref('show_context_menu'))
		{
			//Mostramos el menu

			$('gr-separator').hidden = false;
			$('gr-context-menu').hidden = false;
			$('gr-subscribe-feed-link').hidden = !gContextMenu.getLinkURI();
			$('gr-subscribe-feed-page').hidden  = getWebNavigation().currentURI.spec.indexOf('http', 0) == -1;

			if (gContextMenu.isTextSelected)
			{
				$('gr-search_google_reader').hidden = false;
				$('gr-search_google_reader').label = Gr.properties.getFormattedString('searchGoogleReader',['"' + getSelectedText(20) + '"...']);
			}
			else
			{
				$('gr-search_google_reader').hidden = true;
				$('gr-search_google_reader').label = "";
			}
		}
		else
		{
			//Ocultamos el menu

			$('gr-separator').hidden = true;
			$('gr-context-menu').hidden = true;
		}
	},

	/**
	* Abre una url en la ventana padre
	*
	* @param string url
	* @return void
	*/

	openUrl: function(url)
	{

		var parentWindow = null;

	    if(window.opener)
	    {
	        parentWindow = (window.opener.opener) ? window.opener.opener : window.opener;
	    }
		else
		{
			parentWindow = window;
		}

        switch(GrPref.prefs.getIntPref('context_menu_links'))
		{
			case gGrClickCurrentTab:
				parentWindow.getBrowser().loadURI(url);
			break;

			case gGrClickNewTab:
				parentWindow.getBrowser().selectedTab = parentWindow.getBrowser().addTab(url);
			break;

			case gGrClickUnfocusedTab :
				parentWindow.getBrowser().addTab(url);
			break;

			case gGrClickNewWindow:
				parentWindow.open(url);
			break;

		}
	},

	/**
	* Comprueba si una URL ya esta abierta en alguna pestaña
	*
	* @param string url
	* @param bool select
	* @return void
	*/

	openAndReuseOneTabPerURL: function(url, select)
	{
		var num = gBrowser.browsers.length;
		for (var i = 0; i < num; i++)
		{
			if (url ==  gBrowser.getBrowserAtIndex(i).currentURI.spec)
			{
				gBrowser.mTabContainer.selectedIndex = i;
				return true;
			}
		}

		return false;
	},

	/**
	 * Función para cambiar el icono
	 *
	 * @param string Estado a cambiar
	 * @return void
	 */

	changeIcon: function(status)
	{
		var extension = '.png';

		if(status == gGrLoadingStatus)
		{
			extension = '.gif';
		}

		$('gr-icon').setAttribute("status", status);
		$('gr-icon').setAttribute("src", "chrome://google_reader_notifier/skin/" + GrPref.prefs.getCharPref('icon_theme') + "/" + status + extension);
	},

	/**
	* Función para mostrar las preferencias
	*
	* @return void
	*/

	showPreferences: function()
	{
		window.openDialog('chrome://google_reader_notifier/content/preferences.xul','gr-config-dialog','chrome,centerscreen,modal=yes,resizable=yes');
	},

	/**
	* Función para mostrar la información
	*
	* @return void
	*/

	showAbout: function()
	{
		window.openDialog('chrome://google_reader_notifier/content/about.xul','gr-about-dialog','chrome,centerscreen,modal=yes,resizable=no');
	}
}
